import { AbstractHub } from 'duplex-message';
import { IAbstractHubOptions } from 'duplex-message';
import { IFn } from 'duplex-message';
import { IHandlerMap } from 'duplex-message';
import { IpcMain } from 'electron';
import { IpcMainEvent } from 'electron';
import { IpcRenderer } from 'electron';
import { IpcRendererEvent } from 'electron';
import { IProgress } from 'duplex-message';
import { IRequest } from 'duplex-message';
import { IResponse } from 'duplex-message';
import { setConfig } from 'duplex-message';
import { WebContents } from 'electron';

declare abstract class ElectronMessageHub extends AbstractHub {
    protected readonly _channelName: string;
    protected readonly _ipc: IpcMain | IpcRenderer;
    constructor(options: IElectronBaseMessageHubOptions, className: string);
    protected sendMessage(target: WebContents | IpcRenderer, msg: IRequest | IResponse | IProgress): void;
    protected onMessage(evt: IpcMainEvent | IpcRendererEvent, msg: any): Promise<void>;
    destroy(): void;
}

declare interface IElectronBaseMessageHubOptions extends IAbstractHubOptions {
    /** ipc channel name used under the hood, default: message-hub */
    channelName?: string;
    /**
     * process type
     */
    type: 'browser' | 'renderer';
}

declare type IElectronMessageHubOptions = Omit<IElectronBaseMessageHubOptions, 'type'>;

export declare class MainMessageHub extends ElectronMessageHub {
    constructor(options?: IElectronMessageHubOptions);
    emit<ResponseType = unknown>(target: WebContents, method: string, ...args: any[]): Promise<ResponseType>;
    on(target: WebContents | '*', handlerMap: IFn | IHandlerMap): void;
    on(target: WebContents | '*', methodName: string, handler: IFn): void;
    off(target: WebContents | '*', methodName?: string): void;
    /** shared MainMessageHub instance */
    static get shared(): MainMessageHub;
}

export declare class RendererMessageHub extends ElectronMessageHub {
    constructor(options?: IElectronMessageHubOptions);
    emit<ResponseType = unknown>(method: string, ...args: any[]): Promise<ResponseType>;
    on(handlerMap: IFn | IHandlerMap): void;
    on(methodName: string, handler: IFn): void;
    off(methodName?: string): void;
    /** shared MainMessageHub instance */
    static get shared(): RendererMessageHub;
}

export { setConfig }

export { }
